package com.yaoyicloud.service.impl;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.atomic.AtomicInteger;

import cn.hutool.core.util.IdUtil;
import com.yaoyicloud.config.CommonDataCache;
import com.yaoyicloud.config.FilerepoProperties;
import com.yaoyicloud.config.ReportPathManager;
import com.yaoyicloud.constant.enums.ModuleType;
import com.yaoyicloud.entity.ReportGenerationResult;
import com.yaoyicloud.render.InterestConflictsRender;
import com.yaoyicloud.render.ProjectInfoRender;
import com.yaoyicloud.render.PublicRecordRender;

import com.yaoyicloud.render.ServiceProviderInfoRender;
import com.yaoyicloud.render.association.AssociationAttachmentSectionRender;
import com.yaoyicloud.render.foundation.FoundationAttachmentSectionRender;
import com.yaoyicloud.render.foundation.FoundationfinancialInfoRender;
import com.yaoyicloud.render.platform.AntiBriberyRender;
import com.yaoyicloud.render.platform.AttachmentSectionRender;
import com.yaoyicloud.render.platform.BasicInfoRender;
import com.yaoyicloud.render.platform.FinancialInfoRender;
import com.yaoyicloud.render.association.AssociationBasicInfoRender;
import com.yaoyicloud.render.foundation.FoundationBasicInfoRender;
import com.yaoyicloud.render.platform.update.AntiBriberyNewRender;
import com.yaoyicloud.render.platform.update.BasicInfoNewRender;
import com.yaoyicloud.render.platform.update.FinancialInfoNewRender;
import com.yaoyicloud.render.platform.update.PublicRecordNewRender;
import com.yaoyicloud.render.platform.update.TaxNewRender;

import com.yaoyicloud.service.ReportUpdateService;

import com.yaoyicloud.tools.DocxUtil;

import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.springframework.stereotype.Service;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import static com.yaoyicloud.config.SessionInterceptor.SESSION_MAP;

@Slf4j
@Service
@RequiredArgsConstructor
public class ReportUpdateServiceImpl implements ReportUpdateService {
    // 使用ConcurrentHashMap存储relationId与计数器的映射
    private final ConcurrentMap<Long, AtomicInteger> moduleCounters = new ConcurrentHashMap<>();

    private final FilerepoProperties filerepoProperties;
    private final ReportPathManager reportPathManager;
    private final CommonDataCache commonDataCache;

    @SuppressWarnings("checkstyle:ReturnCount")
    @Override
    public ReportGenerationResult createPlusVersionCheckReport(
        String data, byte[] templateBytes, Long relationId,
        ModuleType moduleType)
        throws Exception {

        // 1. 将字节流模版写入临时文件
        File tempTemplateFile = File.createTempFile("template_", ".docx");
        try (FileOutputStream fos = new FileOutputStream(tempTemplateFile)) {
            fos.write(templateBytes);
        }

        // 获取或初始化计数器
        AtomicInteger counter = moduleCounters.computeIfAbsent(
            relationId,
            k -> new AtomicInteger(1));
        String sessionId = SESSION_MAP.get(relationId.toString());
        String reportPath = null;

        reportPath = processModule(moduleType, data, templateBytes, sessionId, relationId, counter);
        // 处理指定模块
        reportPathManager.addReportPath(sessionId, reportPath);
        counter.incrementAndGet();

        String mergedReportPath = null;
        boolean isLastModule = reportPath.contains("attachmentSection");

        if (isLastModule) {
            List<String> reportPaths = reportPathManager.getReportPaths(sessionId);
            XWPFDocument targetDoc;
            try (FileInputStream fis = new FileInputStream(reportPaths.get(0))) {
                targetDoc = new XWPFDocument(fis);
            }
            List<XWPFDocument> sourceDocs = new ArrayList<>();
            for (int i = 1; i < reportPaths.size(); i++) {
                try (FileInputStream fis = new FileInputStream(reportPaths.get(i))) {
                    sourceDocs.add(new XWPFDocument(fis));
                }
            }
            String label = relationId + "_" + IdUtil.fastSimpleUUID();
            mergedReportPath = filerepoProperties.getBasePath() + "/" + label + ".docx";
            // 调用合并方法
            DocxUtil.mergeDocx(targetDoc, sourceDocs, mergedReportPath, relationId);
            // 清除缓存
            moduleCounters.remove(relationId);
        }

        return new ReportGenerationResult(reportPath, mergedReportPath, isLastModule);
    }

    /**
     * 处理单个模块的报告生成
     */
    @SuppressWarnings("checkstyle:ParameterNumber")
    private String processModule(ModuleType moduleType, String data,
        byte[] templateBytes, String sessionId, Long relationId, AtomicInteger counter) throws Exception {
        String reportPath;

        switch (moduleType) {
            case ANTIBRIBERY:
                reportPath = new AntiBriberyRender(sessionId, filerepoProperties, commonDataCache)
                    .renderDocx(data, templateBytes, String.valueOf(relationId));
                break;
            case ANTIBRIBERY_NEW:
                reportPath = new AntiBriberyNewRender(sessionId, filerepoProperties, commonDataCache)
                    .renderDocx(data, templateBytes, String.valueOf(relationId), counter);
                break;
            case PLATFORM_COMPANY_BASICINFO:
                reportPath = new BasicInfoRender(sessionId, filerepoProperties, commonDataCache)
                    .renderDocx(data, templateBytes, String.valueOf(relationId));
                break;
            case PLATFORM_COMPANY_ATTACHMENTSECTION:
                reportPath = new AttachmentSectionRender(sessionId, filerepoProperties, commonDataCache)
                    .renderDocx(data, templateBytes, String.valueOf(relationId), counter);
                break;
            case ASSOCIATION_ATTACHMENTSECTION:
                reportPath = new AssociationAttachmentSectionRender(sessionId, filerepoProperties, commonDataCache)
                    .renderDocx(data, templateBytes, String.valueOf(relationId));
                break;
            case FOUNDATION_ATTACHMENTSECTION:
                reportPath = new FoundationAttachmentSectionRender(sessionId, filerepoProperties, commonDataCache)
                    .renderDocx(data, templateBytes, String.valueOf(relationId));
                break;
            case COMMON:
                reportPath = new ServiceProviderInfoRender(sessionId, filerepoProperties, commonDataCache)
                    .renderDocx(data, templateBytes, String.valueOf(relationId), counter);
                break;
            case PLATFORM_COMPANY_FINANCIALINFO:
                reportPath = new FinancialInfoRender(sessionId, filerepoProperties, commonDataCache)
                    .renderDocx(data, templateBytes, String.valueOf(relationId));
                break;
            case PLATFORM_COMPANY_FINANCIALINFO_NEW:
                reportPath = new FinancialInfoNewRender(sessionId, filerepoProperties, commonDataCache)
                    .renderDocx(data, templateBytes, String.valueOf(relationId), counter);
                break;
            case FOUNDATION_FINANCIALINFO:
                reportPath = new FoundationfinancialInfoRender(sessionId, filerepoProperties, commonDataCache)
                    .renderDocx(data, templateBytes, String.valueOf(relationId));
                break;
            case PUBLIC_RECORD:
                reportPath = new PublicRecordRender(sessionId, filerepoProperties, commonDataCache)
                    .renderDocx(data, templateBytes, String.valueOf(relationId));
                break;
            case PUBLIC_RECORD_NEW:
                reportPath = new PublicRecordNewRender(sessionId, filerepoProperties, commonDataCache)
                    .renderDocx(data, templateBytes, String.valueOf(relationId), counter);
                break;
            case PLATFORM_COMPANY_BASICINFO_NEW:
                reportPath = new BasicInfoNewRender(sessionId, filerepoProperties, commonDataCache)
                    .renderDocx(data, templateBytes, String.valueOf(relationId), counter);
                break;
            case FOUNDATION_BASICINFO:
                reportPath = new FoundationBasicInfoRender(sessionId, filerepoProperties, commonDataCache)
                    .renderDocx(data, templateBytes, String.valueOf(relationId));
                break;
            case ASSOCIATION_BASICINFO:
                reportPath = new AssociationBasicInfoRender(sessionId, filerepoProperties, commonDataCache)
                    .renderDocx(data, templateBytes, String.valueOf(relationId));
                break;
            case PROJECT:
                reportPath = new ProjectInfoRender(sessionId, filerepoProperties, commonDataCache)
                    .renderDocx(data, templateBytes, String.valueOf(relationId));
                break;
            case INTERESTCONFLICTS:
                reportPath = new InterestConflictsRender(sessionId, filerepoProperties, commonDataCache)
                    .renderDocx(data, templateBytes, String.valueOf(relationId));
                break;
            case TAX_NEW:
                reportPath = new TaxNewRender(sessionId, filerepoProperties, commonDataCache)
                    .renderDocx(data, templateBytes, String.valueOf(relationId), counter);
                break;
            default:
                throw new UnsupportedOperationException("Unsupported module type: " + moduleType);
        }
        return reportPath;
    }

}
